@@ -51,7 +51,7 @@ static int open(struct net_device *dev)
 #endif
 #endif
 
-	ret = socket_init(wg);
+	ret = socket_init(wg, wg->incoming_port);
 	if (ret < 0)
 		return ret;
 	mutex_lock(&wg->device_update_lock);
@@ -105,7 +105,7 @@ static int stop(struct net_device *dev)
 	}
 	mutex_unlock(&wg->device_update_lock);
 	skb_queue_purge(&wg->incoming_handshakes);
-	socket_uninit(wg);
+	socket_reinit(wg, NULL, NULL);
 	return 0;
 }
 
@@ -124,7 +124,7 @@ static netdev_tx_t xmit(struct sk_buff *skb, struct net_device *dev)
 		goto err;
 	}
 
-	peer = routing_table_lookup_dst(&wg->peer_routing_table, skb);
+	peer = allowedips_lookup_dst(&wg->peer_allowedips, skb);
 	if (unlikely(!peer)) {
 		ret = -ENOKEY;
 		net_dbg_skb_ratelimited("%s: No peer is configured for %pISc\n", dev->name, skb);
@@ -204,22 +204,23 @@ static void destruct(struct net_device *dev)
 {
 	struct wireguard_device *wg = netdev_priv(dev);
 
-	mutex_lock(&wg->device_update_lock);
 	rtnl_lock();
 	list_del(&wg->device_list);
 	rtnl_unlock();
-	peer_remove_all(wg); /* The final references are cleared in the below calls to destroy_workqueue. */
+	mutex_lock(&wg->device_update_lock);
 	wg->incoming_port = 0;
+	socket_reinit(wg, NULL, NULL);
+	peer_remove_all(wg); /* The final references are cleared in the below calls to destroy_workqueue. */
 	destroy_workqueue(wg->handshake_receive_wq);
 	destroy_workqueue(wg->handshake_send_wq);
 	packet_queue_free(&wg->decrypt_queue, true);
 	packet_queue_free(&wg->encrypt_queue, true);
 	destroy_workqueue(wg->packet_crypt_wq);
-	routing_table_free(&wg->peer_routing_table, &wg->device_update_lock);
+	rcu_barrier_bh(); /* Wait for all the peers to be actually freed. */
+	allowedips_free(&wg->peer_allowedips, &wg->device_update_lock);
 	ratelimiter_uninit();
 	memzero_explicit(&wg->static_identity, sizeof(struct noise_static_identity));
 	skb_queue_purge(&wg->incoming_handshakes);
-	socket_uninit(wg);
 	free_percpu(dev->tstats);
 	free_percpu(wg->incoming_handshakes_worker);
 	if (wg->have_creating_net_ref)
@@ -272,7 +273,7 @@ static int newlink(struct net *src_net, struct net_device *dev, struct nlattr *t
 	skb_queue_head_init(&wg->incoming_handshakes);
 	pubkey_hashtable_init(&wg->peer_hashtable);
 	index_hashtable_init(&wg->index_hashtable);
-	routing_table_init(&wg->peer_routing_table);
+	allowedips_init(&wg->peer_allowedips);
 	cookie_checker_init(&wg->cookie_checker, wg);
 	INIT_LIST_HEAD(&wg->peer_list);
 	wg->device_update_gen = 1;
