@@ -28,7 +28,7 @@ static void packet_send_handshake_initiation(struct wireguard_peer *peer)
 	peer->last_sent_handshake = get_jiffies_64();
 	up_write(&peer->handshake.lock);
 
-	net_dbg_ratelimited("%s: Sending handshake initiation to peer %Lu (%pISpfsc)\n", peer->device->dev->name, peer->internal_id, &peer->endpoint.addr);
+	net_dbg_ratelimited("%s: Sending handshake initiation to peer %llu (%pISpfsc)\n", peer->device->dev->name, peer->internal_id, &peer->endpoint.addr);
 
 	if (noise_handshake_create_initiation(&packet, &peer->handshake)) {
 		cookie_add_mac_to_packet(&packet, sizeof(packet), peer);
@@ -52,7 +52,8 @@ void packet_send_queued_handshake_initiation(struct wireguard_peer *peer, bool i
 		peer->timer_handshake_attempts = 0;
 
 	/* First checking the timestamp here is just an optimization; it will
-	 * be caught while properly locked inside the actual work queue. */
+	 * be caught while properly locked inside the actual work queue.
+	 */
 	if (!time_is_before_jiffies64(peer->last_sent_handshake + REKEY_TIMEOUT))
 		return;
 
@@ -66,7 +67,7 @@ void packet_send_handshake_response(struct wireguard_peer *peer)
 {
 	struct message_handshake_response packet;
 
-	net_dbg_ratelimited("%s: Sending handshake response to peer %Lu (%pISpfsc)\n", peer->device->dev->name, peer->internal_id, &peer->endpoint.addr);
+	net_dbg_ratelimited("%s: Sending handshake response to peer %llu (%pISpfsc)\n", peer->device->dev->name, peer->internal_id, &peer->endpoint.addr);
 	peer->last_sent_handshake = get_jiffies_64();
 
 	if (noise_handshake_create_response(&packet, &peer->handshake)) {
@@ -110,7 +111,8 @@ static inline unsigned int skb_padding(struct sk_buff *skb)
 	/* We do this modulo business with the MTU, just in case the networking layer
 	 * gives us a packet that's bigger than the MTU. Since we support GSO, this
 	 * isn't strictly neccessary, but it's better to be cautious here, especially
-	 * if that code ever changes. */
+	 * if that code ever changes.
+	 */
 	unsigned int last_unit = skb->len % skb->dev->mtu;
 	unsigned int padded_size = (last_unit + MESSAGE_PADDING_MULTIPLE - 1) & ~(MESSAGE_PADDING_MULTIPLE - 1);
 
@@ -149,6 +151,7 @@ static inline bool skb_encrypt(struct sk_buff *skb, struct noise_keypair *keypai
 		skb_checksum_help(skb);
 
 	/* Only after checksumming can we safely add on the padding at the end and the header. */
+	skb_set_inner_network_header(skb, 0);
 	header = (struct message_data *)skb_push(skb, sizeof(struct message_data));
 	header->header.type = cpu_to_le32(MESSAGE_DATA);
 	header->key_idx = keypair->remote_index;
@@ -173,7 +176,7 @@ void packet_send_keepalive(struct wireguard_peer *peer)
 		skb_reserve(skb, DATA_PACKET_HEAD_ROOM);
 		skb->dev = peer->device->dev;
 		skb_queue_tail(&peer->staged_packet_queue, skb);
-		net_dbg_ratelimited("%s: Sending keepalive packet to peer %Lu (%pISpfsc)\n", peer->device->dev->name, peer->internal_id, &peer->endpoint.addr);
+		net_dbg_ratelimited("%s: Sending keepalive packet to peer %llu (%pISpfsc)\n", peer->device->dev->name, peer->internal_id, &peer->endpoint.addr);
 	}
 
 	packet_send_staged_packets(peer);
@@ -183,7 +186,8 @@ void packet_send_keepalive(struct wireguard_peer *peer)
 static inline void skb_free_null_queue(struct sk_buff *first)
 {
 	struct sk_buff *skb, *next;
-	skb_walk_null_queue_safe (first, skb, next)
+
+	skb_walk_null_queue_safe(first, skb, next)
 		dev_kfree_skb(skb);
 }
 
@@ -193,7 +197,7 @@ static void packet_create_data_done(struct sk_buff *first, struct wireguard_peer
 	bool is_keepalive, data_sent = false;
 
 	timers_any_authenticated_packet_traversal(peer);
-	skb_walk_null_queue_safe (first, skb, next) {
+	skb_walk_null_queue_safe(first, skb, next) {
 		is_keepalive = skb->len == message_data_len(0);
 		if (likely(!socket_send_skb_to_peer(peer, skb, PACKET_CB(skb)->ds) && !is_keepalive))
 			data_sent = true;
@@ -237,20 +241,17 @@ void packet_encrypt_worker(struct work_struct *work)
 	bool have_simd = chacha20poly1305_init_simd();
 
 	while ((first = ptr_ring_consume_bh(&queue->ring)) != NULL) {
-		bool dead = false;
+		enum packet_state state = PACKET_STATE_CRYPTED;
 
-		skb_walk_null_queue_safe (first, skb, next) {
+		skb_walk_null_queue_safe(first, skb, next) {
 			if (likely(skb_encrypt(skb, PACKET_CB(first)->keypair, have_simd)))
 				skb_reset(skb);
 			else {
-				queue_enqueue_per_peer(&PACKET_PEER(first)->tx_queue, first, PACKET_STATE_DEAD);
-				dead = true;
+				state = PACKET_STATE_DEAD;
 				break;
 			}
 		}
-		if (unlikely(dead))
-			continue;
-		queue_enqueue_per_peer(&PACKET_PEER(first)->tx_queue, first, PACKET_STATE_CRYPTED);
+		queue_enqueue_per_peer(&PACKET_PEER(first)->tx_queue, first, state);
 	}
 	chacha20poly1305_deinit_simd(have_simd);
 }
@@ -303,8 +304,9 @@ void packet_send_staged_packets(struct wireguard_peer *peer)
 
 	/* After we know we have a somewhat valid key, we now try to assign nonces to
 	 * all of the packets in the queue. If we can't assign nonces for all of them,
-	 * we just consider it a failure and wait for the next handshake. */
-	skb_queue_walk (&packets, skb) {
+	 * we just consider it a failure and wait for the next handshake.
+	 */
+	skb_queue_walk(&packets, skb) {
 		PACKET_CB(skb)->ds = ip_tunnel_ecn_encap(0 /* No outer TOS: no leak. TODO: should we use flowi->tos as outer? */, ip_hdr(skb), skb);
 		PACKET_CB(skb)->nonce = atomic64_inc_return(&key->counter.counter) - 1;
 		if (unlikely(PACKET_CB(skb)->nonce >= REJECT_AFTER_MESSAGES))
@@ -323,18 +325,21 @@ out_nokey:
 	noise_keypair_put(keypair);
 
 	/* We orphan the packets if we're waiting on a handshake, so that they
-	 * don't block a socket's pool. */
-	skb_queue_walk (&packets, skb)
+	 * don't block a socket's pool.
+	 */
+	skb_queue_walk(&packets, skb)
 		skb_orphan(skb);
 	/* Then we put them back on the top of the queue. We're not too concerned about
 	 * accidently getting things a little out of order if packets are being added
 	 * really fast, because this queue is for before packets can even be sent and
-	 * it's small anyway. */
+	 * it's small anyway.
+	 */
 	spin_lock_bh(&peer->staged_packet_queue.lock);
 	skb_queue_splice(&packets, &peer->staged_packet_queue);
 	spin_unlock_bh(&peer->staged_packet_queue.lock);
 
 	/* If we're exiting because there's something wrong with the key, it means
-	 * we should initiate a new handshake. */
+	 * we should initiate a new handshake.
+	 */
 	packet_send_queued_handshake_initiation(peer, false);
 }
